<?php
namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;


class MinhasTarefasFactory {

    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function getTarefasListadas($listaOperadores, $filtros) {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NRSEQCONTRATO, LC.CDCLIENTE,
                    TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                    LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, C.NRCONTRATOFCT, CT.NMRAZSOCCLIE,
                                    CASE CA.IDSITUACAOAGEND 
                                    WHEN '1' THEN 'Pendente'
                                    WHEN '2' THEN 'Em andamento'
                                    WHEN '3' THEN 'Cancelado'
                                    WHEN '4' THEN 'Concluído'
                                    WHEN '5' THEN 'Atrasado'
                                    ELSE NULL END AS SITUACAO
                                FROM FCM_CHECKLISTAGEND CA
                                JOIN FCM_LOCALCLI LC
                                ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                                JOIN FCM_LOCALCLI LCP
                                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                                JOIN CLIENTE CT
                                ON CT.CDCLIENTE = LC.CDCLIENTE
                                JOIN FCM_CHECKLIST CHK
                                ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                                JOIN OPERADOR F
                                ON F.CDOPERADOR = CA.CDOPERADOR
                                JOIN FCT_CONTRATO C
                                ON C.NRSEQCONTRATO = CA.NRSEQCONTRATO
                                WHERE CA.CDOPERADOR IN ({$listaOperadores})".$filtros."
                             ORDER BY CA.DTHORAAGEND";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getFotosTarefas($parametros) {

        $sql = "SELECT FCM_FOTOTAREFAAGEND.NRFOTOTAREFAAGEND, FCM_FOTOTAREFAAGEND.DSFOTOTAREFAAGEND, FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND
                    FROM FCM_FOTOTAREFAAGEND
                    JOIN FCM_TAREFAAGEND 
                    ON FCM_TAREFAAGEND.NRTAREFAAGEND = FCM_FOTOTAREFAAGEND.NRTAREFAAGEND
                    JOIN FCM_CHECKLISTAGEND 
                    ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_TAREFAAGEND.NRCHECKLISTAGEND
                    WHERE FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function excluirFotoTarefas($parametros) {
        
        $sql = "DELETE FROM FCM_FOTOTAREFAAGEND
                    WHERE NRFOTOTAREFAAGEND = :NRFOTOTAREFAAGEND";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);          
    }

    //Funções da Tela Minhas Tarefas
    public function nextTaskCard ($parametros) {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NRSEQCONTRATO, LC.CDCLIENTE,
        TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
        LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, C.NRCONTRATOFCT, CT.NMRAZSOCCLIE,
                  CASE CA.IDSITUACAOAGEND WHEN '1' THEN 'Pendente'
                  WHEN '2' THEN 'Em andamento'
                  WHEN '5' THEN 'Atrasado'
                  ELSE NULL END AS SITUACAO
              FROM FCM_CHECKLISTAGEND CA
              JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
              JOIN FCM_LOCALCLI LCP
                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
              JOIN CLIENTE CT
                ON CT.CDCLIENTE = LC.CDCLIENTE
              JOIN FCM_CHECKLIST CHK
                ON CA.NRCHECKLIST = CHK.NRCHECKLIST
              JOIN OPERADOR F
                ON F.CDOPERADOR = CA.CDOPERADOR
              JOIN FCT_CONTRATO C
                ON C.NRSEQCONTRATO = CA.NRSEQCONTRATO
              WHERE CA.IDSITUACAOAGEND IN (1,2,5)
              AND (F.CDOPERADOR IN (SELECT  A.CDOPERADOR
                                     FROM FCM_FUNCIONARIOSHIERARQ A
                                     JOIN FCM_HIERARQUIAFUNC H 
                                       ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                                    WHERE A.NRHIERARQUIAFUNC = (SELECT NRHIERARQUIAFUNC
                                                                  FROM FCM_HIERARQUIAFUNC
                                                                 WHERE IDATIVO = 'S')
                              AND A.CDOPERADOR IN (SELECT CDOPERADOR
                                                                  FROM FCM_FUNCIONARIOSHIERARQ
                                                              START WITH CDOPERADOR     = :CDOPERADOR
                                                              CONNECT BY PRIOR CDOPERADOR = CDOPERADORPAI))
               OR (CA.CDOPERINCLUSAO = :CDOPERADOR))
              ORDER BY CA.DTHORAAGEND";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        return $std->fetchAll();
    }
    
    public function itensDetailsTaskForVistoria($parametros) {
      $resposta = array(
        "observation" => [],
        "criterion" => []
      );
      $sql = "SELECT DISTINCT TA.DSOBSTAREFAAGEND
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
                 ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
              WHERE TA.DSOBSTAREFAAGEND IS NOT NULL
                AND 
                    CA.NRCHECKLISTAGEND = (SELECT NRCHECKLISTAGEND
                                             FROM FCM_GERENCIAMENTOCHK
                                            WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

      $std = $this->connection->prepare($sql);

      $std->execute($parametros);
      $resposta['observation'] =  $std->fetchAll();
      
      $sql = "SELECT DISTINCT TF.NMTAREFA, TA.IDREALIZADO
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
                 ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
               JOIN FCM_TAREFACHK T
                 ON T.NRTAREFACHK = TA.NRTAREFACHK
               JOIN FCM_TAREFA TF
                 ON T.NRTAREFA = TF.NRTAREFA
              WHERE CA.NRCHECKLISTAGEND = (SELECT NRCHECKLISTAGEND 
                                             FROM FCM_GERENCIAMENTOCHK
                                            WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

      $std = $this->connection->prepare($sql);

      $std->execute($parametros);
      $resposta['criterion'] =  $std->fetchAll();
      return $resposta;
    }


    // Update para alterar o status da tarefa
    // Quando iniciar a tarefa , passar o valor '2' / Quando finalizar, passar o valor '4'  / Quando reportar, passar o valor 3
    // DSJUSTCANCELAMENTO = Deve ser preenchido somente quando for reportar. Para as outras ações, deve ser passado "NULL"
    public function upStatusTask ($parametros) {

      $sql = "UPDATE FCM_CHECKLISTAGEND
                SET IDSITUACAOAGEND = :IDSITUACAOAGEND,
                DTULTATU = SYSDATE,
                DSJUSTCANCELAMENTO = :DSJUSTCANCELAMENTO
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

      $std = $this->connection->prepare($sql);
      $std->execute($parametros);
    }
    
    public function upStatusVistoryCanc ($parametros) {

      $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                DTULTATU = SYSDATE
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

      $std = $this->connection->prepare($sql);
      $std->execute($parametros);
    }
    
    // Update de finalização direta na tela de Minhas Tarefas
    public function upFinishTask($parametros) {

      $sql = "UPDATE FCM_TAREFAAGEND
                SET IDREALIZADO = 'S',
                DTULTATU = SYSDATE
              WHERE NRCHECKLISTAGEND = ".$parametros;

      $std = $this->connection->prepare($sql);
      $std->execute();
    }

    //Funções da Tela Detalhamento da Tarefa

    // Update de finalização do detalhamento
    // IDREALIZADO = Somente dos checks que foram marcados
    // (Nesse caso, deve ser fazer um ForEach pegando somente os que foram marcados e updatando 1 por 1 )
    public function upFinishDetailsTaks($parametros) {
      $sql = " UPDATE FCM_TAREFAAGEND
                SET IDREALIZADO = 'S',
                DTULTATU = SYSDATE,
                DSOBSTAREFAAGEND = :DSOBSTAREFAAGEND
              WHERE NRTAREFAAGEND = :NRTAREFAAGEND
              AND NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

      $std = $this->connection->prepare($sql);
      $std->execute($parametros);
    }

    // Funções da Tela de Detalhamento

    //Função que retorna as informações no card superior com o detalhamento da tarefa
    public function itensDetailsTask($parametros) {

      $sql = "SELECT TA.NRTAREFAAGEND, TA.NRCHECKLISTAGEND, TA.CDOPERADOR, TO_CHAR(TA.DTHORATAREFAAGEND, 'DD/MM HH24:MI') DTHORATAREFAAGEND,
              TA.NRTAREFACHK, TA.IDREALIZADO, TF.NMTAREFA
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
               ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
               JOIN FCM_TAREFACHK T
               ON T.NRTAREFACHK = TA.NRTAREFACHK
               JOIN FCM_TAREFA TF
               ON T.NRTAREFA = TF.NRTAREFA
               WHERE CA.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

      $std = $this->connection->prepare($sql);

      $std->execute($parametros);
      return $std->fetchAll();
    }

    public function cardDetailsTask($parametros) {

      $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND,
        TO_CHAR(CA.DTHORAAGEND, 'DD/MM HH24:MI') DTHORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
        LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, CA.IDOBRIGAOBSERVACAO,
                  CASE CA.IDSITUACAOAGEND WHEN '1' THEN 'Pendente'
                  WHEN '2' THEN 'Em andamento'
                  WHEN '5' THEN 'Atrasado'
                  ELSE NULL END AS IDSITUACAOAGEND
              FROM FCM_CHECKLISTAGEND CA
              JOIN FCM_LOCALCLI LC
              ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
              JOIN FCM_LOCALCLI LCP
              ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
              JOIN FCM_CHECKLIST CHK
              ON CA.NRCHECKLIST = CHK.NRCHECKLIST
              JOIN OPERADOR F
              ON F.CDOPERADOR = CA.CDOPERADOR
              WHERE CA.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

      $std = $this->connection->prepare($sql);

      $std->execute($parametros);
      return $std->fetchAll();
    }

    public function statusCheck($param) {

      $sql = "SELECT COUNT(IDSITUACAOAGEND)
               FROM FCM_CHECKLISTAGEND
               WHERE IDSITUACAOAGEND = '2'
               AND CDOPERADOR = :CDOPERADOR";

      $std = $this->connection->prepare($sql);

      $std->execute($param);
      return $std->fetchAll();
    }

}
?>