/* eslint-disable */
import {
    Card, Text, CheckboxMultiple, Textarea, Button, AlertService, LoadingService, Form, Modal, ModalService, SelectMultiple, DateRange,
} from '@zeedhi/common';
import {
    IDictionary, Http, Metadata, IEventParam, Router, SimpleDatasource,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { Client } from '@pusher/push-notifications-web';
import { Utilities } from '@/util/Utilities';
import { getRouteParams } from '../router';

export class ListagemVistoriasController {
    public quantidadeAtividades: Number = 0;

    public trueFalseValue: String = 'Não';

    public idTarefa: IDictionary = {
        IDSITUACAOAGEND: [],
        IDSITUACAOGERCHK: [],
    };

    public selected: IDictionary = {};

    public dataBase: IDictionary = [];

    public modalVisualizarImagens!: Modal;

    public modalFiltrarVistorias!: Modal;

    public modalInformacoes!: Modal;

    public nrGerenciamentoChk: any = '';

    public vistoria: any = ''; 

    public contrato: any = '';

    public cliente: any = '';

    public tarefa: any = '';

    public local: any = '';

    public data: any = '';

    public funcionario: any = '';

    public supervisor: any = '';

    public situacao: any = '';

    public carregarCardsVistorias({ component }: any) {
        component.datasource.setFilter({
            TIPO: 1,
        });
    }

    public async criarModais() {
        this.modalVisualizarImagens = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00002_000_vistoria/modal/fm_00002_000_visualizar_imagens' });
        this.modalFiltrarVistorias = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00002_000_vistoria/modal/fm_00002_000_filtrar_vistorias' });
        this.modalInformacoes = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00002_000_vistoria/modal/fm_00002_000_dados_vistoria' });
    }

    public destruirModais() {
        this.modalVisualizarImagens?.destroy();
        this.modalFiltrarVistorias?.destroy();
        this.modalInformacoes?.destroy();
    }

    public async exibirModalVisualizarImagens({ component }: any) {
        const nr = component.name.split('_')[1];
        this.nrGerenciamentoChk = nr;
        this.modalVisualizarImagens.show();
    }

    public async exibirModalFiltros() {
        this.modalFiltrarVistorias.show();
    }

    public fecharModalInformacoes() {
        this.modalInformacoes.hide();
    }

    public fecharModalVisualizarImagens() {
        this.modalVisualizarImagens.hide();
    }

    public fecharModalFiltros() {
        this.modalFiltrarVistorias.hide();
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-vistorias');

        iterable.datasource.setFilter({
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            CDOPERADOR: Metadata.getInstance<SelectMultiple>('filtro-operador').value,
            IDSITUACAOGERCHK: Metadata.getInstance<SelectMultiple>('filtro-situacao').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            DTHORAGEND: Metadata.getInstance<DateRange>('data').value,
            TIPO: 3,
        }).finally(() => {
            this.fecharModalFiltros();
        });
    }
    
    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-vistorias');

        Metadata.getInstance<SelectMultiple>('filtro-cliente').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-operador').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-situacao').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-contratos').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        this.carregarCardsVistorias({ component: iterable });
    }

    public async exibirModalInformacoes({ component }: any) {
        LoadingService.show();
        Http.post('/getVistoriasListadas', {
            NRGERENCIAMENTOCHK: component.name.split('_')[1],
            TIPO: 2,
        }).then((response: IDictionary) => {
            const dados = response.data.data[0];

            this.vistoria = dados.NMGERENCIAMENTOCHK;
            this.contrato = (dados.NRCONTRATOFCT === null) ? 'Não há' : dados.NRCONTRATOFCT;
            this.cliente = dados.NMFANTCLIE;
            this.tarefa = dados.NMCHECKLISTAGEND;
            this.local = `${dados.NMLOCALCLIENTE} - ${dados.NMLOCALPAI}`;
            this.data = `${dados.DTAGEND} - ${dados.HORAAGEND}`;
            this.funcionario = dados.NMOPERADOR;
            this.supervisor = dados.SUPERVISOR;
            this.situacao = dados.SITUACAO;

            this.modalInformacoes.show();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public clickCard({ component }: any) {
        Router.replace(`/Detalhes_Vistoria/${component.name.split('_')[1]}/${component.name.split('_')[2]}`);
    }

    public carregarFotosVistoria({ component }: any) {
        const nr = this.nrGerenciamentoChk;
        const btnApagar = Metadata.getInstance<Button>('botao-apagar-imagens-vistoria');
        LoadingService.show();
        Http.post('/getFotosVistorias', {
            NRGERENCIAMENTOCHK: nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-vistorias').isVisible = true;
                btnApagar.isVisible = false;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAGERCHK: dados.NRFOTOTAREFAGERCHK,
                        DSFOTOTAREFAGERCHK: dados.DSFOTOTAREFAGERCHK,
                    });
                    btnApagar.isVisible = true;
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async excluirFoto() {
        const carrosel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-vistorias');
        const img = carrosel.iterable.datasource.data;
        const [deleteData] = img.filter((value: any, index: any) => index === carrosel.currentSlide);

        LoadingService.show();
        Http.post('/excluirFotoVistorias', {
            NRFOTOTAREFAGERCHK: deleteData.NRFOTOTAREFAGERCHK,
        }).then(() => {
            const data = img.filter((value: any, index: any) => index !== carrosel.currentSlide);
            carrosel.iterable.datasource.updateData(data);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    // -------------> FUNÇÕES DA TELA DOS CRITÉRIOS <----------------
    public showCriteria({ component }: { component: Card }) {
        const routeParams = getRouteParams();
        const { nr } = routeParams;
        if (Object.keys(this.selected).length) {
            this.dataBase[this.selected.name] = {
                NRGERENCIAMENTOCHK: nr,
                NRATIVIDADECHK: this.selected.name.substr(9),
                valuesChkbox: Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `${dados}`).join(','),
                valueObservation: Metadata.getInstance<Textarea>('textAreaObservation').value,
            };
            this.selected.cssClass = 'card3 cardAct cardAct2 card-disable';
        }
        component.cssClass = 'card-background card3 cardAct cardAct2';
        this.selected = component;
        const title: any = component!.children[0]!;
        const titleAux: any = title!.children[1]!;
        const header: any = titleAux!.children[0]!;

        Metadata.getInstance<Text>('subTitleHeader').text = header.text;
        Metadata.getInstance<Text>('textSelectConformities').text = 'Marque os campos de não conformidades:';
        Metadata.getInstance<Textarea>('textAreaObservation').isVisible = true;

        Http.post('/getTask', {
            NRATIVIDADECHK: component.name.substr(9),
            NRGERENCIAMENTOCHK: nr,
        }).then((response: IDictionary) => {
            response.data.dataset.criterias.forEach((criterias: IDictionary) => {
                this.idTarefa.IDSITUACAOGERCHK = criterias.IDSITUACAOGERCHK;
                this.idTarefa.IDSITUACAOAGEND = criterias.IDSITUACAOAGEND;
                const validaObs = criterias.IDOBRIGAOBSERVACAO;

                if (criterias.IDSITUACAOGERCHK === '3' || criterias.IDSITUACAOAGEND !== '4') {
                    this.activateButtonFinalize();// desativa o botão finalize, torna apenas visualização
                } else {
                    this.activateButton();
                }
                if (validaObs !== 'S') {
                    Metadata.getInstance<Textarea>('textAreaObservation').clearValidations();
                } else {
                    Metadata.getInstance<Textarea>('textAreaObservation').addValidation('required', {});
                }
                const chkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple');
                if (chkbox.datasource instanceof SimpleDatasource) {
                    chkbox!.datasource!.items = criterias.CRITERIOS;
                }
                if (Object.keys(this.dataBase).includes(component.name)) {
                    chkbox.value = this.dataBase[component.name].valuesChkbox.replaceAll('\'').split(',');
                    Metadata.getInstance<Textarea>('textAreaObservation').value = this.dataBase[component.name].valueObservation;
                } else {
                    chkbox.value = criterias.CRITERIOSATIVOS;
                    Metadata.getInstance<Textarea>('textAreaObservation').value = criterias.DSOBSERVACAO;
                }
            });
        });
    }

    public activateButton() {
        const saveButton = Metadata.getInstance<Button>('saveButton');

        saveButton.disabled = false;
    }

    public showDetailsSurvey() {
        const routeParams = getRouteParams();
        const { nr, local } = routeParams;
        Metadata.getInstance<Text>('textTitleLocal').text = local.split('|')[0];
        Http.post('/showDetailsSurvey', { nr }).then((response: IDictionary) => {
            this.quantidadeAtividades = response.data.dataset.detailsSurvey.length;
            response.data.dataset.detailsSurvey.forEach((dados: IDictionary) => {
                Metadata.getInstance<IterableCarousel>('iCarousel').iterable.datasource.data.push({
                    NRTAREFA: dados.NRATIVIDADECHK,
                    NAME: dados.NMATIVIDADE,
                });
            });
        });
    }

    public saveCriterias({ component }: { component: Button }) {
        const routeParams = getRouteParams();
        const { nr } = routeParams;
        let id: any;
        let message: any;
        const formObs = Metadata.getInstance<Form>('Form-observation');

        this.dataBase[this.selected.name] = {
            NRGERENCIAMENTOCHK: nr,
            NRATIVIDADECHK: this.selected.name.substr(9),
            valuesChkbox: Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `'${dados}'`).join(','),
            valueObservation: Metadata.getInstance<Textarea>('textAreaObservation').value,
        };
        const dataBase = Object.keys(this.dataBase).map((key: string) => this.dataBase[key]);

        const selectedCriteria = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple').value.map((dados: any) => `${dados}`);
        // console.log(selectedCriteria);
        if (component.name === 'saveButton') {
            id = 2;
            message = 'Atividade salva com sucesso!';
            const areaComentatio = Metadata.getInstance<Textarea>('textAreaObservation');
            if (Object.keys(selectedCriteria).length === 0) {
                console.log(Object.keys(selectedCriteria).length);
                if (areaComentatio.value === '') {
                    Metadata.getInstance<Textarea>('textAreaObservation').value = 'Vistoria realizada, sem não-conformidades';
                }
            }
            if (Object.keys(this.dataBase).length === this.quantidadeAtividades) {
                Metadata.getInstance<Button>('finalizeButton').disabled = false;
            }
        } else {
            id = 3;
            message = 'Vistoria realizada com sucesso!';
            this.activateButtonFinalize();
        }

        if (Utilities.validateForm(formObs)) {
            LoadingService.show();
            Http.post('/upSurveyCriterias', [id, dataBase]).then(() => {
                Utilities.alertMessage(message, 'sucess');
                if (id === 3) {
                    Router.replace('/fm_00002_000_vistoria');
                }
            }).catch((error: any) => {
                if (component.name !== 'saveButton') {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public activateButtonFinalize() {
        const chkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-multiple');
        const tArea = Metadata.getInstance<Textarea>('textAreaObservation');
        const saveButton = Metadata.getInstance<Button>('saveButton');
        chkbox.readonly = true;
        tArea.readonly = true;
        saveButton.disabled = true;
    }

    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Aguardando Vistoria') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>AGUARDANDO VISTORIA</span>";
        }
        if (component.text === 'Avaliado') {
            component.text = "<span  style='background-color: #dee2e6; color: #2b9348; font-size: 12px;'>A V A L I A D O</span>";
        }
        if (component.text === 'Em andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 12px;'>EM ANDAMENTO</span>";
        }
        if (component.text === 'Cancelado') {
            component.text = "<span  style='background-color:#dee2e6; color: black; font-size: 12px;'>C A N C E L A D O</span>";
        }
        if (component.text === 'Tarefa Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>TAREFA PENDENTE</span>";
        }
    }

    public subscribeSurveyNotifications() {
        const idInst: string = process.env.VUE_APP_PUSH_NOTIFICATION_INSTANCE_ID!;
        const beamsClient = new Client({
            instanceId: idInst,
        });
        Notification.requestPermission()
            .then((result) => {
                if (result === 'granted') {
                    beamsClient.start()
                        .then(() => {
                            const nrUser = window.localStorage.getItem('UserLogado');
                            const devInterest = `survey-${nrUser}`;
                            beamsClient.addDeviceInterest(devInterest)
                                .then(() => {
                                    AlertService.show({
                                        name: 'basic-alert',
                                        text: 'Agora você receberá notificações sobre suas vistorias',
                                        top: true,
                                        left: false,
                                        timeout: 3000,
                                        color: 'success',
                                    });
                                })
                                .catch((err: any) => {
                                    AlertService.show({
                                        name: 'basic-alert',
                                        text: err,
                                        top: true,
                                        left: false,
                                        timeout: 3000,
                                        color: 'error',
                                    });
                                });
                        })
                        .catch((err: any) => {
                            AlertService.show({
                                name: 'basic-alert',
                                text: err,
                                top: true,
                                left: false,
                                timeout: 3000,
                                color: 'error',
                            });
                        });
                } else {
                    AlertService.show({
                        name: 'basic-alert',
                        text: 'Você não permitiu que as notificações sejam enviadas. Você pode tentar ativar novamente se mudar ideia.',
                        top: true,
                        left: false,
                        timeout: 3000,
                        color: 'warning',
                    });
                }
            })
            .catch((err) => {
                AlertService.show({
                    name: 'basic-alert',
                    text: err,
                    top: true,
                    left: false,
                    timeout: 3000,
                    color: 'error',
                });
            });
    }
}
