/* eslint-disable */
import { Utilities } from '@/util/Utilities';
import {
  ModalService, Modal, IModal, DialogService, LoadingService,
} from '@zeedhi/common';
import {
  Router, Http, Loader, Metadata,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';

export class QrCodeController {
  private qrCodeModal!: Modal;

  public url: String = '';

  public testeModal!: Modal;

  public dadosCard: any;

  public telaVistoria = Loader.getInstance('ListagemVistoriasController');

  public decodeQrCode({ decodeString }: any) {
    this.url = decodeString;
    this.buscaOperador();
  }

  public buscaOperador() {
    LoadingService.show();
    Http.post('/buscaOperador', {
      ATIVO: 'S',
      NRLOCALCLIENTE: this.url,
      CDOPERADOR: window.localStorage.getItem('CDOPERADOR'),
    }).then((response: any) => {
      this.montaCard(response);
      Utilities.showSuccessMessage();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public montaCard(response: any) {
    const T = 'Detalhes_Tarefa';
    const V = 'Detalhes_Vistoria';
    if (response.data.length === 4 && response.data === 'NL{}') {
      const texto = 'Operador não cadastrado na hierarquia, favor verificar.';
      this.mensagem(texto);
    } else if (response.data.length === 1) {
      const texto = 'Não há ' + (response.data.find((n: string) => n === 'PAI') ? 'vistorias' : ' tarefas') + ' pendentes para este local';
      this.mensagem(texto);
    } else if (response.data.length === 2 && response.data.find((n: string) => n === 'PAI')) {
      const novaUrl = `${V}/${response.data[0].NRGERENCIAMENTOCHK}/${response.data[0].NMLOCALCLIENTE} | ${response.data[0].NMLOCALPAI} `;
      Router.replace(novaUrl);
      Utilities.showSuccessMessage();
      this.fecharModal();
    } else if (response.data.length === 2 && response.data.find((n: string) => n === 'FILHO')) {
      const novaUrl = `${T}/${response.data[0].NRCHECKLISTAGEND}`;
      Router.replace(novaUrl);
      Utilities.showSuccessMessage();
      this.fecharModal();
    } else if (response.data.length > 2 && response.data.find((n: string) => n === 'PAI')) {
      const teste = '';
      this.fecharModal();
      Router.replace('Vistorias');
      //this.telaVistoria.filter(teste);
    } else if (response.data.length > 2 && response.data.find((n: string) => n === 'FILHO')) {
      this.dadosCard = response.data;
      this.fecharModal();
      this.abrirModalTarefas();
    }
  }

  public montaGrid() {
    const grid = Metadata.getInstance<TekGrid>('tek-grid-escolha-tarefas');
    this.dadosCard.forEach((dados: any) => {
      grid.datasource.data.push({
        NRCHECKLISTAGEND: dados.NRCHECKLISTAGEND,
        DATAHORA: dados.DATAHORA,
        NMCHECKLISTAGEND: dados.NMCHECKLISTAGEND,
      });
    });
  }

  public redirecionar({ row }: any) {
    const novaUrl = `Detalhes_Tarefa/${row.NRCHECKLISTAGEND}`;
    Router.replace(novaUrl);
    this.fecharModalTarefas();
  }

  public async abrirModalTarefas() {
    this.testeModal = await ModalService.createFromJson('modal_qr_code', true);
    this.testeModal.show();
  }

  public fecharModalTarefas() {
    this.testeModal.hide();
    this.testeModal.destroy();
  }

  public createModal() {
    const modal: IModal = {
      name: 'qrCodeModal',
      persistent: true,
      fullscreen: true,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'closeButtonHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cols: 12,
              children: [
                {
                  name: 'titleText',
                  component: 'ZdText',
                  text: 'Escaneie o QR Code',
                  cssClass: 'text-center',
                  style: {
                    'margin-bottom': '15px',
                    'font-weight': 'bold',
                    'font-size': '20px',
                  },
                },
              ],
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'qrCodeModal',
            },
          ],
        },
        {
          name: 'rowQrCode',
          component: 'ZdRow',
          children: [
            {
              name: 'qrcode',
              component: 'QrCodeReader',
              events: {
                decode: this.decodeQrCode.bind(this),
              },
            },
          ],
        },
      ],
    };
    this.qrCodeModal = ModalService.create(modal);
    this.qrCodeModal.show();
  }

  public fecharModal() {
    this.qrCodeModal.destroy();
    this.qrCodeModal.hide();
    DialogService.hide();
  }

  public mensagem(texto: string) {
    DialogService.show({
      name: 'basic-dialog',
      title: 'Atenção',
      text: `${texto}`,
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sair',
          events: {
            click: '{{QrCodeController.fecharModal}}',
          },
        },
      ],
    });
  }
}
